#!/bin/bash

quote_json() {
    sed -e 's|\\|\\\\|g' -e 's|\"|\\\"|g'
}

page_append()
{
    if [ -z "$1" ]; then
        echo "$2"
    elif [ -z "$2" ]; then
        echo "$1"
    else
        echo "$1,$2"
    fi
}

FOUND_DB_TITLE_JSON="Przywróć lub utwórz bazę danych MariaDB 10"
FOUND_DB_TITLE_JAVASCRIPT="Przywróć lub utwórz bazę danych MariaDB 10"
DECIDE_RESTORE_DESC="Wybierz dowolną z następujących akcji:"
RESTORE_DESC="Przywróć istniejącą bazę danych"
CREATE_NEW_DESC="Utwórz nową bazę danych"
WIZARD_SET_DATA_TITLE_JSON="Konfiguruj MariaDB 10"
WIZARD_SET_DATA_TITLE_JAVASCRIPT="Konfiguruj MariaDB 10"
DB_NEW_PASSWORD="Nowe hasło"
INVALID_USER_PASSWORD="Hasło nie spełnia bieżących reguł siły. Hasło musi mieć co najmniej 10 znaków i musi zawierać wielkie i małe litery, cyfry oraz znaki specjalne; nie może zawierać popularnych haseł ani nazwy użytkownika."
DB_CONFIRM_PASSWORD="Potwierdź hasło"
PORT="Port"
BAD_PORT="Numer portu powinien być od 1024 do 65535."

MYSQLDB_ESSENTIAL_TABLE="user db host tables_priv columns_priv func"

ChkDB() {
    local mysqlPath=$1
    for tbl in ${MYSQLDB_ESSENTIAL_TABLE}; do
        if [ ! -f ${mysqlPath}/mysql/${tbl}.frm -o ! -f ${mysqlPath}/mysql/${tbl}.MYI -o ! -f ${mysqlPath}/mysql/${tbl}.MYD ]; then
            return 1
            break;
        fi
    done
    return 0
}

ChkExecEnv() {
    local mysqlSubPath=

    if [ "yes" = "$IS_USBSTATION" ]; then
        mysqlSubPath="@sharebin/@database/mariadb10"
        volRegExp='/volumeUSB[1-9]/usbshare'
    else
        mysqlSubPath="@database/mariadb10"
        volRegExp='/volume[1-9]'
    fi

    for volume in `grep -e " $volRegExp " /proc/mounts | awk '{ print $2 }'`; do
        if [ -d "${volume}/${mysqlSubPath}" ]; then
            return $(ChkDB "${volume}/${mysqlSubPath}")
        fi
    done
    return 1
}

RESTORE_PAGE=$(/bin/cat<<EOF
{
    "step_title": "${FOUND_DB_TITLE_JSON}",
    "items": [{
        "type": "singleselect",
        "desc": "${DECIDE_RESTORE_DESC}",
        "subitems": [{
            "key": "pkgwizard_restore",
            "desc": "${RESTORE_DESC}",
            "defaultValue": true
        }, {
            "key": "pkgwizard_create",
            "desc": "${CREATE_NEW_DESC}",
            "defaultValue": false
        }, {
            "key": "pkgwizard_found_old_data",
            "hidden": true,
            "defaultValue": false
        }]
    }]
}
EOF
)

jsFunction=$(/bin/cat<<EOF
    function findStepByTitle(wizardDialog, title) {
        for (var i = wizardDialog.customuiIds.length - 1 ; i >= 0 ; i--) {
            var step = wizardDialog.getStep(wizardDialog.customuiIds[i]);
            if (title === step.headline) {
                return step;
            }
        }
        return null;
    }
    function isRestoreChecked(wizardDialog) {
        var restoreStep = findStepByTitle(wizardDialog, "${FOUND_DB_TITLE_JAVASCRIPT}");
        if (!restoreStep) {
            return false;
        } else {
            return restoreStep.items.items[1].checked;
        }
    }
EOF
)

getActiveate()
{
    ACTIVAETE=$(/bin/cat<<EOF
{
    ${jsFunction}
    var currentStep = arguments[0];
    var wizardDialog = currentStep.owner;
    var restoreStep = findStepByTitle(wizardDialog, "${FOUND_DB_TITLE_JAVASCRIPT}");
    var settingStep = findStepByTitle(wizardDialog, "${WIZARD_SET_DATA_TITLE_JAVASCRIPT}");
    var checked = isRestoreChecked(wizardDialog);
    settingStep.items.items[0].setVisible(!checked);
    settingStep.items.items[1].setVisible(!checked);
}
EOF
)
    echo "$ACTIVAETE" | quote_json
}

getPasswordValidator()
{
    validator=$(/bin/cat<<EOF
{
    ${jsFunction}
    var password = arguments[0];
    var currentStep = arguments[2];
    if(isRestoreChecked(currentStep.owner)) {
        return true;
    } else {
        return -1 !== password.search("(?=.*[A-Z])(?=.*[a-z])(?=.*[0-9])(?=.*[^A-Za-z0-9])(?=.{10,})") && ! password.includes("root");
    }
}
EOF
)
    echo "$validator" | quote_json
}

getPasswordConfirmValidator()
{
    validator=$(/bin/cat<<EOF
{
    ${jsFunction}
    var passwordConfirm = arguments[0];
    var currentStep = arguments[2];
    if(isRestoreChecked(currentStep.owner)) {
        return true;
    } else {
        return passwordConfirm === currentStep.items.items[0].value;
    }
}
EOF
)
    echo "$validator" | quote_json
}

getPortValidator()
{
    validator=$(/bin/cat<<EOF
{
    var strPort = arguments[0];
    var port = parseInt(strPort, 10);
    return -1 !== strPort.search("^[0-9]+$") && 1024 <= port && port <= 65535;
}
EOF
)
    echo "$validator" | quote_json
}

SETTING_PAGE=$(/bin/cat<<EOF
{
    "step_title": "${WIZARD_SET_DATA_TITLE_JSON}",
    "invalid_next_disabled_v2": true,
    "activeate": "$(getActiveate)",
    "items": [{
        "type": "password",
        "subitems": [{
            "key": "pkgwizard_new_root_password",
            "desc": "${DB_NEW_PASSWORD}",
            "invalidText": "${INVALID_USER_PASSWORD}",
            "validator": {
                "fn": "$(getPasswordValidator)"
            }
        }, {
            "desc": "${DB_CONFIRM_PASSWORD}",
            "validator": {
                "fn": "$(getPasswordConfirmValidator)"
            }
        }]
    }, {
        "type": "textfield",
        "subitems": [{
            "key": "pkgwizard_port",
            "desc": "${PORT}",
            "defaultValue": "3307",
            "invalidText": "${BAD_PORT}",
            "validator": {
                "allowBlank": false,
                "fn": "$(getPortValidator)"
            }
        }]
    }]
}
EOF
)

main() {
    local install_page=""
    if ChkExecEnv; then
        install_page=$(page_append "$install_page" "$RESTORE_PAGE")
    fi
    install_page=$(page_append "$install_page" "$SETTING_PAGE")
    echo "[$install_page]" > "${SYNOPKG_TEMP_LOGFILE}"
}
main "$@"
